#pragma once

#include "FBullCowGame.h"
#include <map>
#define TMap std::map // To make syntax unreal friendly


FBullCowGame::FBullCowGame()
{
	Reset();
}

int32 FBullCowGame::GetMaxTries() const { 
	TMap<int32, int32> maxTries = { {3, 4}, {4, 6}, {5, 8}, {6, 9}, {7, 12}, {8, 15} };
	return maxTries[MyHiddenWord.length()];
}

int32 FBullCowGame::GetCurrentTry() const {	return MyCurrentTry; }

int32 FBullCowGame::GetHiddenWordLength() const { return MyHiddenWord.length(); }

bool FBullCowGame::isLowercase(FString Guess) const 
{
	bool isLowercase = true;
	for (auto letter : Guess) {
		if (!islower(letter)) {
			isLowercase = false;
			break;
		}
	}
	return isLowercase;
}

bool FBullCowGame::isIsogram(FString Guess) const 
{ 
	bool isIsogram = true;
	// Create map with all alphabet letters hashed to a boolean in a default false state
	TMap<char, bool> LetterSeen;
	// Create for loop to run through guessed word

	for (auto letter : Guess)
	{
		letter = tolower(letter);
		// Check letter against corresponding boolean
		if (!LetterSeen[letter]) {
			// if false, change to true and continue to next letter
			LetterSeen[letter] = true;
		}
		else {
			// if true finish loop and return false
			isIsogram = false;
			break;
		}

	}
	// return true if loop is finished without any true statement encounters 
	return isIsogram;
}

bool FBullCowGame::IsGameWon() const
{
	return bGameWon;
}

EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{
	// if the guess isn't an isogram
	if (!isIsogram(Guess))
	{
		return EGuessStatus::NOT_Isogram;
	}
	// if the guess isn't all lowercare
	else if (!isLowercase(Guess))
	{
		return EGuessStatus::Not_Lowercase;
	}
	// If guess length is wrong
	else if (GetHiddenWordLength() != Guess.length())
	{
		return EGuessStatus::Wrong_Length;
	}
	else
	{
		return EGuessStatus::OK;
	}
}

// Receives a VALID guess, increments turn, and returns count.
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;

	// setup a return vaiable
	FBullCowCount bullCowCount;

	// Assusming same length as guess
	int32 WordLenght = MyHiddenWord.length();

	// loop through all letters in the hideen word
	for (int32 MHWChar = 0; MHWChar < WordLenght; MHWChar++) {
		// compare against the guess
		for (int32 GChar = 0; GChar < WordLenght; GChar++) {
			if (MyHiddenWord[MHWChar] == tolower(Guess[GChar]) && MHWChar == GChar) {
				bullCowCount.Bulls++;
			}
			else if (MyHiddenWord[MHWChar] == tolower(Guess[GChar])) {
				bullCowCount.Cows++;
			}
		}
	}

	if (bullCowCount.Bulls == GetHiddenWordLength()) {
		bGameWon = true;
	}

	return bullCowCount;
}

void FBullCowGame::Reset()
{
	bGameWon = false;

	const FString HIDEN_WORD = "house"; // MUST be an isogram
	MyHiddenWord = HIDEN_WORD;
	
	MyCurrentTry = 1;

	return;
}