/*
	This is the console executable, that makes use of the BullCow Class.
	Acts as the view in a MVC pattern, and is responsible for all user
	interaction. For game logic see FBullCowGame class.
*/

#include <iostream>
#include <string>
#include "FBullCowGame.h"

using FText = std::string;
using int32 = int;

// Functions declared so that other funcions recognize them regardless of where the definition is.
void PrintIntro();
FText GetValidGuess();
void PlayGame();
bool AskToPlayAgain();
void PrintGameSummary();

FBullCowGame BCGame; // Instantiates FBullCowGame

// Application entry point
int main()
{
	bool bPlayAgain;
	do {
		PrintIntro();
		PlayGame();
		PrintGameSummary();
		bPlayAgain = AskToPlayAgain();
	} while (bPlayAgain);
	
	return 0;
}

void PrintIntro()
{
	// Game start/introduction
	std::cout << "\n\nWelcome to cows and bulls!" << std::endl;
	std::cout << "Can you guess the " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram?\n\n";
	return;
}

bool AskToPlayAgain()
{
	std::cout << "Do you want to play again? (y/n)" << std::endl;
	FText answer = "";
	getline(std::cin, answer);
	
	return (answer[0] == 'y' || answer[0] == 'Y');
}

void PlayGame()
{
	BCGame.Reset();

	int32 MaxTries = BCGame.GetMaxTries();
	std::cout << "You have " << MaxTries << " tries to solve the mistery word!" << std::endl;

	// Loop asking for guesses or while the game is not won
	// and there are still tries remaining
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries) {
		FString Guess = GetValidGuess(); 

		// TODO Submit valid guess to the game
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
		
		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << ". Cows = " << BullCowCount.Cows << "." << "\n";
		std::cout << "Your guess: " << Guess << "\n\n";

	}
	return;
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon()) {
		std::cout << "\n\nCongratulations! Your guess was correct so you win the game!\n\n";
	}
	else {
		std::cout << "\n\nYou have reached the maximum try limit, you lose\n\n";
	}
}

// Loop continually until user gives valid guess
FText GetValidGuess()
{
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	FString Guess = "";

	do {
		int32 CurrentGuess = BCGame.GetCurrentTry();

		// Get guess from player
		std::cout << "Try " << CurrentGuess << " of " << BCGame.GetMaxTries() << ". Take a guess: ";
		getline(std::cin, Guess);

		Status = BCGame.CheckGuessValidity(Guess);

		switch (Status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Please enter a word with all lowercase letters.\n";
			break;
		case EGuessStatus::NOT_Isogram:
			std::cout << "Please enter word without repeat letters.\n";
			break;
		default:
			Status = EGuessStatus::OK;
			break;
		}
		std::cout << std::endl;
	} while (Status != EGuessStatus::OK); // Keep looping until we get no errors

	return Guess;
}

